# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2015-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing analyze subcommand

"""
from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)

import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.evaluation_steps as evaluation
import bigmler.tests.dataset_advanced_steps as dataset_adv


def setup_module():
    """Setup for the module

    """
    common_setup_module()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()

class TestAnalyze:
    """Testing bigmler analyze"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def test_scenario1(self):
        """
        Scenario: Successfully building k-fold cross-validation from dataset:
            Given I create BigML dataset uploading train "<data>" file in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I create BigML <kfold>-fold cross-validation
            And I check that the <kfold>-datasets have been created
            And I check that the <kfold>-models have been created
            And I check that the <kfold>-fold cross-validation has been created
            Then the evaluation file is like "<json_evaluation_file>"
        """
        print(self.test_scenario1.__doc__)
        headers = ["data", "output", "kfold", "json_evaluation_file"]
        examples = [
            ['data/iris.csv', 'scenario_a_1/evaluation', '2',
             'check_files/evaluation_kfold.json']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_dataset(
                self, data=example["data"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_pred.i_create_kfold_cross_validation(
                self, k_folds=example["kfold"])
            test_pred.i_check_create_kfold_datasets(self, example["kfold"])
            test_pred.i_check_create_kfold_models(self, example["kfold"])
            test_pred.i_check_create_kfold_cross_validation(
                self, example["kfold"])
            evaluation.then_the_evaluation_file_is_like(
                self, example["json_evaluation_file"])

    def test_scenario2(self):
        """
        Scenario: Successfully building feature selection from dataset:
            Given I create BigML dataset uploading train "<data>" file in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I create BigML feature selection <kfold>-fold cross-validations improving "<metric>"
            And I check that the <kfold>-datasets have been created
            And I check that the <kfold>-models have been created
            And I check that all the <kfold>-fold cross-validations have been created
            Then the best feature selection is "<selection>", with "<metric>" of <metric_value>
            And I generate a report from the output directory
            And a symlink file is generated in the reports directory
        """
        print(self.test_scenario2.__doc__)
        headers = ["data", "output", "kfold", "metric", "selection",
                   "metric_value"]
        examples = [
            ['data/iris_2f.csv', 'scenario_a_2/evaluation', '2', 'accuracy',
             'petal width', '100.00%'],
            ['data/iris_2f.csv', 'scenario_a_3/evaluation', '2', 'phi',
             'petal width', '1']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_dataset(
                self, data=example["data"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_pred.i_create_kfold_cross_validation_metric(
                self, k_folds=example["kfold"], metric=example["metric"])
            test_pred.i_check_create_kfold_datasets(self, example["kfold"])
            test_pred.i_check_create_kfold_models(self, example["kfold"])
            test_pred.i_check_create_all_kfold_cross_validations(
                self, example["kfold"])
            test_pred.i_check_feature_selection(
                self, example["selection"], example["metric"],
                example["metric_value"])
            test_pred.i_generate_report(self)
            test_pred.is_symlink(self)

    def test_scenario3(self):
        """
        Scenario: Successfully building feature selection from dataset setting objective:
            Given I create BigML dataset uploading train "<data>" file in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I create BigML feature selection <kfold>-fold cross-validations for "<objective>" improving "<metric>"
            And I check that the <kfold>-datasets have been created
            And I check that the <kfold>-models have been created
            And I check that all the <kfold>-fold cross-validations have been created
            Then the best feature selection is "<selection>", with "<metric>" of <metric_value>
        """
        print(self.test_scenario3.__doc__)
        headers = ["data", "objective", "output", "kfold", "metric",
                   "selection", "metric_value"]
        examples = [
            ['data/iris_2f.csv', '0', 'scenario_a_5/evaluation', '2',
             'r_squared', 'species', '0.352845'],
            ['data/iris_2f.csv', '0', 'scenario_a_8/evaluation', '2',
             'mean_squared_error', 'species', '0.475200']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_dataset(
                self, data=example["data"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_pred.i_create_kfold_cross_validation_objective(
                self, k_folds=example["kfold"], objective=example["objective"],
                metric=example["metric"])
            test_pred.i_check_create_kfold_datasets(self, example["kfold"])
            test_pred.i_check_create_kfold_models(self, example["kfold"])
            test_pred.i_check_create_all_kfold_cross_validations(
                self, example["kfold"])
            test_pred.i_check_feature_selection(
                self, example["selection"], example["metric"],
                example["metric_value"])

    def test_scenario4(self):
        """
        Scenario: Successfully building feature selection from filtered dataset setting objective:
            Given I create BigML dataset uploading train "<data>" file in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I filter out field "<field>" from dataset and log to "<output_dir>"
            And I check that the new dataset has been created
            And I create BigML feature selection <kfold>-fold cross-validations for "<objective>" improving "<metric>"
            And I check that the <kfold>-datasets have been created
            And I check that the <kfold>-models have been created
            And I check that all the <kfold>-fold cross-validations have been created
            Then the best feature selection is "<selection>", with "<metric>" of <metric_value>
        """
        print(self.test_scenario4.__doc__)
        headers = ["data", "field", "objective", "output", "output_dir",
                   "kfold", "metric", "selection", "metric_value"]
        examples = [
            ['data/iris_2fd.csv', 'sepal length', 'species',
             'scenario_a_6/evaluation', 'scenario_a_6', '2', 'recall',
             'petal width', '100.00%']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_dataset(
                self, data=example["data"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            dataset_adv.i_filter_field_from_dataset(
                self, field=example["field"], output_dir=example["output_dir"])
            test_pred.i_check_create_new_dataset(self)
            test_pred.i_create_kfold_cross_validation_objective(
                self, k_folds=example["kfold"], objective=example["objective"],
                metric=example["metric"])
            test_pred.i_check_create_kfold_datasets(self, example["kfold"])
            test_pred.i_check_create_kfold_models(self, example["kfold"])
            test_pred.i_check_create_all_kfold_cross_validations(
                self, example["kfold"])
            test_pred.i_check_feature_selection(
                self, example["selection"], example["metric"],
                example["metric_value"])

    def test_scenario5(self):
        """
        Scenario: Successfully building nodes threshold analysis from dataset file:
            Given I create BigML dataset uploading train "<data>" file in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I create BigML nodes analysis from dataset file from <min_nodes> to <max_nodes> by <nodes_step> with <kfold>-cross-validation improving "<metric>"
            And I check that the <kfold>-datasets have been created
            And I check that the <kfold>-models have been created
            And I check that all the <kfold>-fold cross-validations have been created
            Then the best node threshold is "<node_threshold>", with "<metric>" of <metric_value>
        """
        print(self.test_scenario5.__doc__)
        headers = ["data", "output", "min_nodes", "max_nodes", "nodes_step",
                   "kfold", "metric", "node_threshold", "metric_value"]
        examples = [
            ['data/iris.csv', 'scenario_a_4/evaluation', '3', '14', '2', '2',
             'precision', '9', '94.71%']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_dataset(
                self, data=example["data"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_pred.i_create_nodes_analysis_from_dataset_file(
                self, min_nodes=example["min_nodes"],
                max_nodes=example["max_nodes"],
                nodes_step=example["nodes_step"],
                k_fold=example["kfold"],
                metric=example["metric"])
            test_pred.i_check_create_kfold_datasets(self, example["kfold"])
            test_pred.i_check_create_kfold_models(self, example["kfold"])
            test_pred.i_check_create_all_kfold_cross_validations(
                self, example["kfold"])
            test_pred.i_check_node_threshold(
                self, example["node_threshold"], example["metric"],
                example["metric_value"])

    def test_scenario6(self):
        """
            Scenario: Successfully building feature selection from dataset excluding features:
                Given I create BigML dataset uploading train "<data>" file in "<output>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I create BigML feature selection <kfold>-fold cross-validations excluding "<features>" with separator "<args_separator>" improving "<metric>"
                And I check that the <kfold>-datasets have been created
                And I check that the <kfold>-models have been created
                And I check that all the <kfold>-fold cross-validations have been created
                Then the best feature selection is "<selection>", with "<metric>" of <metric_value>
        """
        print(self.test_scenario6.__doc__)
        headers = ["data", "output", "kfold", "features", "args_separator",
                   "metric", "selection", "metric_value"]
        examples = [
            ['data/iris.csv', 'scenario_a_7/evaluation', '2',
             'petal length!sepal width', '!', 'accuracy', 'petal width',
             '95.33%']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_dataset(
                self, data=example["data"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_pred.i_create_kfold_cross_validation_separator_metric_no_fields(
                self, k_folds=example["kfold"], features=example["features"],
                args_separator=example["args_separator"],
                metric=example["metric"])
            test_pred.i_check_create_kfold_datasets(self, example["kfold"])
            test_pred.i_check_create_kfold_models(self, example["kfold"])
            test_pred.i_check_create_all_kfold_cross_validations(
                self, example["kfold"])
            test_pred.i_check_feature_selection(
                self, example["selection"], example["metric"],
                example["metric_value"])

    def test_scenario7(self):
        """
        Scenario: Successfully building feature selection for a category from dataset:
            Given I create BigML dataset uploading train "<data>" file with attributes "<attributes>" in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I create BigML feature selection <kfold>-fold cross-validations improving "<metric>" for category "<category>"
            And I check that the <kfold>-datasets have been created
            And I check that the <kfold>-models have been created
            And I check that all the <kfold>-fold cross-validations have been created
            Then the best feature selection is "<selection>", with "<metric>" of <metric_value>
        """
        print(self.test_scenario7.__doc__)
        headers = ["data", "attributes", "output", "kfold", "metric",
                   "category", "selection", "metric_value"]
        examples = [
            ['data/spam.csv', 'data/spam_attributes.json',
             'scenario_a_9/evaluation', '2', 'recall', 'spam',
             'Message', '58.69%']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_dataset_with_attributes(
                self, data=example["data"], attributes=example["attributes"],
                output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_pred.i_create_kfold_cross_validation_metric_category(
                self, k_folds=example["kfold"], metric=example["metric"],
                category=example["category"])
            test_pred.i_check_create_kfold_datasets(self, example["kfold"])
            test_pred.i_check_create_kfold_models(self, example["kfold"])
            test_pred.i_check_create_all_kfold_cross_validations(
                self, example["kfold"])
            test_pred.i_check_feature_selection(
                self, example["selection"], example["metric"],
                example["metric_value"])

    def test_scenario8(self):
        """
        Scenario: Successfully building a new dataset from an existing one and analyzing it
            Given I create a BigML dataset from "<data>" and store logs in "<output_dir>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I create a new BigML dataset using the specs in JSON file "<new_fields>" and a model with "<model_fields>"
            And I check that the new dataset has been created
            And I check that the model has been created
            And I create BigML nodes analysis from <min_nodes> to <max_nodes> by <nodes_step> with <kfold>-cross-validation improving "<metric>"
            And I check that the <kfold>-datasets have been created
            And I check that the <kfold>-models have been created
            And I check that all the <kfold>-fold cross-validations have been created
            Then the best node threshold is "<node_threshold>", with "<metric>" of <metric_value>
        """
        print(self.test_scenario8.__doc__)
        headers = ["data", "output_dir", "new_fields", "field", "model_fields",
                   "min_nodes", "max_nodes", "nodes_step", "kfold", "metric",
                   "node_threshold", "metric_value"]
        examples = [
            ['data/iris.csv', 'scenario_a_10', 'data/new_fields2.json',
             'outlier?', 'outlier?,species', '3', '14', '2', '2', 'precision',
             '5', '98.21%']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            dataset_adv.i_create_dataset(
                self, data=example["data"], output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            dataset_adv.i_create_dataset_new_fields(
                self, json_file=example["new_fields"],
                model_fields=example["model_fields"])
            test_pred.i_check_create_new_dataset(self)
            test_pred.i_check_create_model(self)
            test_pred.i_create_nodes_analysis(
                self, min_nodes=example["min_nodes"],
                max_nodes=example["max_nodes"],
                nodes_step=example["nodes_step"],
                k_fold=example["kfold"],
                metric=example["metric"])
            test_pred.i_check_create_kfold_datasets(self, example["kfold"])
            test_pred.i_check_create_kfold_models(self, example["kfold"])
            test_pred.i_check_create_all_kfold_cross_validations(
                self, example["kfold"])
            test_pred.i_check_node_threshold(
                self, example["node_threshold"], example["metric"],
                example["metric_value"])

    def test_scenario9(self):
        """
        Scenario: Successfully building random fields analysis from dataset:
            Given I create BigML dataset uploading train "<data>" file in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I create BigML random fields analysis with <kfold>-cross-validation improving "<metric>"
            And I check that the <kfold>-datasets have been created
            And I check that the <kfold>-random trees have been created
            And I check that all the <kfold>-fold cross-validations have been created
            Then the best random candidates number is "<random_candidates>", with "<metric>" of <metric_value>
        """
        print(self.test_scenario9.__doc__)
        headers = ["data", "output", "kfold", "metric", "random_candidates",
                   "metric_value"]
        examples = [
            ['data/iris.csv', 'scenario_a_11/evaluation', '2', 'precision',
             '4', '96.09%']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_dataset(self, data=example["data"],
                output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_pred.i_create_random_analysis(
                self, k_fold=example["kfold"], metric=example["metric"])
            test_pred.i_check_create_kfold_datasets(self, example["kfold"])
            test_pred.i_check_create_kfold_random_forest(
                self, example["kfold"])
            test_pred.i_check_create_all_kfold_cross_validations(
                self, example["kfold"])
            test_pred.i_check_random_candidates(
                self, example["random_candidates"], example["metric"],
                example["metric_value"])

    def test_scenario10(self):
        """
        Scenario: Successfully building feature selection from dataset setting objective:
            Given I create BigML dataset uploading train "<data>" file in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I create BigML feature selection <kfold>-fold cross-validation with options "<options>"
            And I check that the <kfold>-datasets have been created
            And I check that the <kfold>-models have been created
            And I check that all the <kfold>-fold cross-validations have been created
            Then the predictions file "<predictions_file>" is like "<estimated_file>"
        """
        print(self.test_scenario10.__doc__)
        headers = ["data", "output", "kfold", "options", "predictions_file",
                   "estimated_file"]
        examples = [
            ['data/iris.csv', 'scenario_a_12/evaluation', '2',
             ' --exclude-features="petal length,sepal length"' +
             ' --predictions-csv',
             'scenario_a_12/test/kfold2_pred/predictions.csv',
             'check_files/analyze_predictions_iris.csv'],
            ['data/iris.csv', 'scenario_a_13/evaluation', '2',
             ' --exclude-features="species,sepal length"' +
             ' --predictions-csv --objective 0',
             'scenario_a_13/test/kfold6_pred/predictions.csv',
             'check_files/analyze_predictions_iris2.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_dataset(
                self, data=example["data"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_pred.i_create_kfold_cross_validation_options(
                self, k_folds=example["kfold"], options=example["options"])
            test_pred.i_check_create_kfold_datasets(self, example["kfold"])
            test_pred.i_check_create_kfold_models(self, example["kfold"])
            test_pred.i_check_create_all_kfold_cross_validations(
                self, example["kfold"])
            test_pred.i_check_predictions_file(
                self, example["predictions_file"],
                example["estimated_file"])


    def test_scenario11(self):
        """
        Scenario: Successfully building feature selection from dataset setting objective:
            Given I create BigML dataset uploading train "<data>" file in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I create BigML feature selection <kfold>-fold cross-validation with options "<options>"
            And I check that the <kfold>-datasets have been created
            And I check that the <kfold>-ensembles have been created
            And I check that all the <kfold>-fold cross-validations have been created
            Then the predictions file "<predictions_file>" is like "<estimated_file>"
        """
        print(self.test_scenario11.__doc__)
        headers = ["data", "output", "kfold", "options", "predictions_file",
                   "estimated_file"]
        examples = [
            ['data/iris.csv', 'scenario_a_14/evaluation', '2', ' --exclude-features="petal length,sepal length" --predictions-csv --number-of-models 2','scenario_a_14/test/kfold2_pred/predictions.csv', 'check_files/analyze_predictions_iris_e.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_dataset(
                self, data=example["data"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_pred.i_create_kfold_cross_validation_options(
                self, k_folds=example["kfold"], options=example["options"])
            test_pred.i_check_create_kfold_datasets(self, example["kfold"])
            test_pred.i_check_create_kfold_ensembles(self, example["kfold"])
            test_pred.i_check_create_all_kfold_cross_validations(
                self, example["kfold"])
            test_pred.i_check_predictions_file(
                self, example["predictions_file"], example["estimated_file"])



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2015-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing Multi-label evaluations

"""
from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)

import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.ml_tst_evaluation_steps as ml_eval
import bigmler.tests.evaluation_steps as evaluation


def setup_module():
    """Setup for the module

    """
    common_setup_module()
    test = TestMLEvaluation()
    test.bigml = {"method": "setup_scenario1"}
    test.setup_scenario1()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()

class TestMLEvaluation:
    """Test multi-label evaluation """

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def setup_scenario1(self):
        """
        Scenario: Successfully building multi-label evaluations from scratch
            Given I create BigML multi-label resources tagged as "<tag>" with "<label_separator>" label separator and "<number_of_labels>" labels uploading train "<data>" file with "<training_separator>" field separator to evaluate and log evaluation in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the models have been created
            And I check that the <number_of_labels> evaluations have been created
            And I check that the evaluation is ready
            Then the evaluation file is like "<json_evaluation_file>"
        """
        print(self.setup_scenario1.__doc__)
        headers = ["tag", "label_separator", "number_of_labels", "data",
                   "training_separator", "output", "json_evaluation_file"]
        examples = [
            ['my_multilabel_e_1', ':', '7', 'data/multilabel.csv', ',',
             'scenario_ml_e1/evaluation', 'check_files/evaluation_ml.json']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            ml_eval.i_create_all_ml_evaluations(
                self, tag=example["tag"],
                label_separator=example["label_separator"],
                number_of_labels=example["number_of_labels"],
                data=example["data"],
                training_separator=example["training_separator"],
                output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_pred.i_check_create_models(self)
            test_pred.i_check_create_evaluations(
                self, number_of_evaluations=example["number_of_labels"])
            ml_eval.i_check_evaluation_ready(self)
            evaluation.then_the_evaluation_file_is_like(
                self, example["json_evaluation_file"])

    def test_scenario2(self):
        """
        Scenario: Successfully building multi-label evaluations from source
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML multi-label resources using source to evaluate and log evaluation in "<output>"
            And I check that the dataset has been created
            And I check that the models have been created
            And I check that the <number_of_labels> evaluations have been created
            And I check that the evaluation is ready
            Then the evaluation file is like "<json_evaluation_file>"
        """
        print(self.test_scenario2.__doc__)
        headers = ["scenario", "kwargs", "number_of_labels", "output",
                   "json_evaluation_file"]
        examples = [
            ['scenario_ml_e1', '{"tag": "my_multilabel_e_1",' +
             ' "data": "data/multilabel.csv", "label_separator": ":",' +
             ' "number_of_labels": 7, "training_separator": ",",' +
             ' "output": "scenario_ml_e1/evaluation"}', '7',
             'scenario_ml_e2/evaluation', 'check_files/evaluation_ml.json']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            ml_eval.i_create_ml_evaluations_from_source(
                self, output=example["output"])
            test_pred.i_check_create_dataset(self)
            test_pred.i_check_create_models(self)
            test_pred.i_check_create_evaluations(
                self, number_of_evaluations=example["number_of_labels"])
            ml_eval.i_check_evaluation_ready(self)
            evaluation.then_the_evaluation_file_is_like(
                self, example["json_evaluation_file"])

    def test_scenario3(self):
        """
        Scenario: Successfully building multi-label evaluations from dataset
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML multi-label resources using dataset to evaluate and log evaluation in "<output>"
            And I check that the models have been created
            And I check that the <number_of_labels> evaluations have been created
            And I check that the evaluation is ready
            Then the evaluation file is like "<json_evaluation_file>"
        """
        print(self.test_scenario3.__doc__)
        headers = ["scenario", "kwargs", "number_of_labels", "output",
                   "json_evaluation_file"]
        examples = [
            ['scenario_ml_e1', '{"tag": "my_multilabel_e_1",' +
             ' "data": "data/multilabel.csv", "label_separator": ":",' +
             ' "number_of_labels": 7, "training_separator": ",",' +
             ' "output": "scenario_ml_e1/evaluation"}', '7',
             'scenario_ml_e3/evaluation', 'check_files/evaluation_ml.json']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            ml_eval.i_create_ml_evaluations_from_dataset(
                self, output=example["output"])
            test_pred.i_check_create_models(self)
            test_pred.i_check_create_evaluations(
                self, number_of_evaluations=example["number_of_labels"])
            ml_eval.i_check_evaluation_ready(self)
            evaluation.then_the_evaluation_file_is_like(
                self, example["json_evaluation_file"])

    def test_scenario4(self):
        """
        Scenario: Successfully building multi-label evaluations from models file
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML multi-label resources using models in file "<models_file>" to evaluate and log evaluation in "<output>"
            And I check that the <number_of_labels> evaluations have been created
            And I check that the evaluation is ready
            Then the evaluation key "<key>" value for the model is greater than <value>
        """
        print(self.test_scenario4.__doc__)
        headers = ["scenario", "kwargs", "models_file", "number_of_labels",
                   "output", "key", "value"]
        examples = [
            ['scenario_ml_e1', '{"tag": "my_multilabel_e_1",' +
             ' "data": "data/multilabel.csv", "label_separator": ":",' +
             ' "number_of_labels": 7, "training_separator": ",",' +
             ' "output": "scenario_ml_e1/evaluation"}',
             'scenario_ml_e1/models', '7', 'scenario_ml_e4/evaluation',
             'average_phi', '0.8180']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            ml_eval.i_create_ml_evaluations_from_models(
                self, models_file=example["models_file"],
                output=example["output"])
            test_pred.i_check_create_evaluations(
                self, number_of_evaluations=example["number_of_labels"])
            ml_eval.i_check_evaluation_ready(self)
            evaluation.i_check_evaluation_key(
                self, key=example["key"], value=example["value"])

    def test_scenario5(self):
        """
        Scenario: Successfully building multi-label evaluations from models retrieved by tag
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML multi-label resources using models tagged as "<tag>" to evaluate and log evaluation in "<output>"
            And I check that the <number_of_labels> evaluations have been created
            And I check that the evaluation is ready
            Then the evaluation key "<key>" value for the model is greater than <value>
        """
        print(self.test_scenario5.__doc__)
        headers = ["scenario", "kwargs", "tag", "number_of_labels", "output",
                   "key", "value"]
        examples = [
            ['scenario_ml_e1', '{"tag": "my_multilabel_e_1",' +
             ' "data": "data/multilabel.csv", "label_separator": ":",' +
             ' "number_of_labels": 7, "training_separator": ",",' +
             ' "output": "scenario_ml_e1/evaluation"}', 'my_multilabel_e_1' ,
             '7', 'scenario_ml_e5/evaluation', 'average_phi', '0.8180']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            ml_eval.i_create_ml_evaluations_from_tagged_models(
                self, tag=example["tag"], output=example["output"])
            test_pred.i_check_create_evaluations(
                self, number_of_evaluations=example["number_of_labels"])
            ml_eval.i_check_evaluation_ready(self)
            evaluation.i_check_evaluation_key(
                self, key=example["key"], value=example["value"])



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2015-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
""" Testing projects

"""
from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)

import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.project_creation_steps as test_project
import bigmler.tests.delete_subcommand_steps as delete


def setup_module():
    """Setup for the module

    """
    common_setup_module()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()

class TestProjects:
    """Testing projects commands"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def test_scenario1(self):
        """
        Scenario: Successfully creating a project and associating resources to it:
            Given I create a BigML source uploading train "<data>" file and associate it to a new project named "<project>" storing results in "<output_dir>"
            And I check that the project has been created
            And I check that the source has been created
            Then the source is associated to the project
        """
        print(self.test_scenario1.__doc__)
        headers = ["data", "project", "output_dir"]
        examples = [
            ['data/iris.csv', 'My new project', 'scenario_p_1']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_project.i_create_source_with_project(
                self, data=example["data"],
                project=example["project"], output_dir=example["output_dir"])
            test_project.i_check_create_project(self)
            test_pred.i_check_create_source(self)
            test_project.check_source_in_project(self)

    def test_scenario2(self):
        """
        Scenario: Successfully associating resources to an existing project:
            Given I create a BigML source uploading train "<data>" file and associate it to a new project named "<project>" storing results in "<output_dir>"
            And I check that the project has been created
            And I check that the source has been created
            And I create a BigML source uploading train "<data>" file and associate it to the last created project id storing results in "<output_dir2>"
            Then the source is associated to the project
        """
        print(self.test_scenario2.__doc__)
        headers = ["data", "project", "output_dir", "output_dir2"]
        examples = [
            ['data/iris.csv', 'My new project', 'scenario_p_2', 'scenario_p_2_1']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_project.i_create_source_with_project(
                self, data=example["data"], project=example["project"],
                output_dir=example["output_dir"])
            test_project.i_check_create_project(self)
            test_pred.i_check_create_source(self)
            test_project.i_create_source_with_project_id(
                self, data=example["data"], output_dir=example["output_dir"])
            test_project.check_source_in_project(self)

    def test_scenario3(self):
        """
        Scenario: Successfully creating resources with no project association:
            Given I create a BigML source from file "<data>" storing results in "<output_dir>"
            And I check that the source has been created
            Then the source has no project association
        """
        print(self.test_scenario3.__doc__)
        headers = ["data", "output_dir"]
        examples = [
            ['data/iris.csv', 'scenario_p_3']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            delete.i_create_source_from_file(
                self, data=example["data"],
                output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            test_project.check_source_in_no_project(self)

    def test_scenario4(self):
        """
        Scenario: Successfully creating and updating project:
            Given I create a BigML project "<project>" and log results in "<output_dir>"
            And I check that the project has been created
            And I update the project params "<params>" to "<values>"
            And I check that the project has been updated
            Then the project params "<params>" are "<values>"
        """
        print(self.test_scenario4.__doc__)
        headers = ["project", "output_dir", "params", "values"]
        examples = [
            ['My new project', 'scenario_p_4',
             ['name', 'category'], ['my_project_name', 1]]]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_project.i_create_project(
                self, project=example["project"],
                output_dir=example["output_dir"])
            test_project.i_check_create_project(self)
            test_project.i_update_project(
                self, params=example["params"],
                values=example["values"])
            test_project.i_check_update_project(self)
            test_project.check_params_values(
                self, params=example["params"],
                values=example["values"])



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2014-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.



import os
import json

from bigml.api import check_resource

from bigmler.tests.common_steps import base_shell_execute
from bigmler.utils import open_mode
from bigmler.tests.world import world, res_filename, ok_, eq_


def i_create_sample(step, options=None, output_dir=None):
    """Step: I create a new sample from the dataset and get the sample using
    options <options> storing logs in <output_dir>
    """
    ok_(options is not None and  output_dir is not None)
    command = ("bigmler sample --dataset " + world.dataset['resource'] +
               " --store --output-dir " + output_dir +
               " " + options)
    base_shell_execute(command, output_dir)


def i_check_create_sample(step):
    """Step: I check that the sample has been created"""
    sample_file = os.path.join(world.directory, "samples")
    message = None
    try:
        with open(sample_file) as handler:
            sample = check_resource(handler.readline().strip(),
                                    world.api.get_sample)
            world.samples.append(sample['resource'])
            world.sample = sample
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_sample_stat(step, check_sample_json=None):
    """Step: the sample contains stat-info like in <check_sample_json>"""
    ok_(check_sample_json is not None)
    check_sample_json = res_filename(check_sample_json)
    message = None
    try:
        sample_json = os.path.join(world.directory, "stat_info.json")
        with open(check_sample_json) as check_sample_file:
            check_sample_contents = check_sample_file.read()
        with open(sample_json) as sample_file:
            sample_file_contents = sample_file.read()
        eq_(check_sample_contents, sample_file_contents,
            msg=f"File contents:\n{sample_file_contents}\n"
                f"Expected contents:\n{check_sample_contents}")
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_sample_file(step, check_sample_file=None):
    """Step: the sample file is like <check_sample_file>"""
    ok_(check_sample_file is not None)
    check_sample_file = res_filename(check_sample_file)
    message = None
    try:
        sample_file = os.path.join(world.directory, "sample.csv")
        with open(check_sample_file, open_mode("r")) as check_handler:
            check_sample_contents = check_handler.read()
        with open(sample_file, open_mode("r")) as sample_handler:
            sample_file_contents = sample_handler.read()

        eq_(check_sample_contents, sample_file_contents,
            msg=f"File contents:\n{sample_file_contents}\n"
                f"Expected contents:\n{check_sample_contents}")
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_sample_json(step, check_sample_file=None):
    """Step: the sample JSON is like the one in <check_sample_file>"""
    ok_(check_sample_file is not None)
    check_sample_file = res_filename(check_sample_file)
    message = None
    try:
        sample_file = os.path.join(world.directory, "stat_info.json")
        with open(check_sample_file, open_mode("r")) as check_handler:
            contents = check_handler.read()
            check_sample_json = json.loads(contents)
        with open(sample_file, open_mode("r")) as sample_handler:
            contents = sample_handler.read()
            sample_file_json = json.loads(contents)
        eq_(check_sample_json, sample_file_json,
            msg=f"File contents:\n{sample_file_json}\n"
                f"Expected contents:\n{check_sample_json}")
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)



# -*- coding: utf-8 -*-
#
# Copyright 2020-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Resources management functions

"""


import sys

import bigml.api

from bigmler.utils import (dated, get_url, log_message, plural, check_resource,
                           check_resource_error, log_created_resources,
                           is_shared)
from bigmler.reports import report
from bigmler.resourcesapi.common import set_basic_model_args, \
    configure_input_fields, update_sample_parameters_args, \
    update_json_args, wait_for_available_tasks


from bigmler.resourcesapi.common import FIELDS_QS


def set_association_args(args, name=None, fields=None,
                         association_fields=None):
    """Return association arguments dict

    """
    if name is None:
        name = args.name
    if association_fields is None:
        association_fields = args.association_fields_

    association_args = set_basic_model_args(args, name)

    if association_fields and fields is not None:
        input_fields = configure_input_fields(fields, association_fields)
        association_args.update(input_fields=input_fields)
    if args.association_k:
        association_args.update({"max_k": args.association_k})
    if args.search_strategy:
        association_args.update({"search_strategy": args.search_strategy})

    association_args = update_sample_parameters_args(association_args, args)

    if 'association' in args.json_args:
        update_json_args(association_args,
                         args.json_args.get('association'), fields)

    return association_args


def create_associations(datasets, association_ids, association_args,
                        args, api=None, path=None,
                        session_file=None, log=None):
    """Create remote associations

    """
    if api is None:
        api = bigml.api.BigML()

    associations = association_ids[:]
    existing_associations = len(associations)
    association_args_list = []
    datasets = datasets[existing_associations:]
    # if resuming and all associations were created,
    # there will be no datasets left
    if datasets:
        if isinstance(association_args, list):
            association_args_list = association_args

        # Only one association per command, at present
        number_of_associations = 1
        message = dated("Creating %s.\n" %
                        plural("association", number_of_associations))
        log_message(message, log_file=session_file,
                    console=args.verbosity)

        query_string = FIELDS_QS
        inprogress = []
        for i in range(0, number_of_associations):
            wait_for_available_tasks(inprogress,
                                     args.max_parallel_associations,
                                     api, "association")
            if association_args_list:
                association_args = association_args_list[i]

            association = api.create_association(
                datasets, association_args, retries=None)
            association_id = check_resource_error( \
                association, "Failed to create association: ")
            log_message("%s\n" % association_id, log_file=log)
            association_ids.append(association_id)
            inprogress.append(association_id)
            associations.append(association)
            log_created_resources( \
                "associations", path, association_id, mode='a')

        if args.verbosity:
            if bigml.api.get_status(association)['code'] != bigml.api.FINISHED:
                try:
                    association = check_resource( \
                        association, api.get_association,
                        query_string=query_string,
                        raise_on_error=True)
                except Exception as exception:
                    sys.exit("Failed to get a finished association: %s" %
                             str(exception))
                associations[0] = association
            message = dated("Association created: %s\n" %
                            get_url(association))
            log_message(message, log_file=session_file,
                        console=args.verbosity)
            if args.reports:
                report(args.reports, path, association)

    return associations, association_ids


def get_associations(association_ids, args, api=None, session_file=None):
    """Retrieves remote associations in its actual status

    """
    if api is None:
        api = bigml.api.BigML()

    association_id = ""
    associations = association_ids
    association_id = association_ids[0]
    message = dated("Retrieving %s. %s\n" %
                    (plural("association", len(association_ids)),
                     get_url(association_id)))
    log_message(message, log_file=session_file, console=args.verbosity)
    # only one association to predict at present
    try:
        query_string = FIELDS_QS
        association = check_resource(association_ids[0], api.get_association,
                                     query_string=query_string,
                                     raise_on_error=True)
    except Exception as exception:
        sys.exit("Failed to get a finished association: %s" % str(exception))
    associations[0] = association

    return associations, association_ids


def set_publish_association_args(args):
    """Set args to publish association

    """
    public_association = {}
    if args.public_association:
        public_association = {"private": False}
        if args.model_price:
            public_association.update(price=args.model_price)
        if args.cpp:
            public_association.update(credits_per_prediction=args.cpp)
    return public_association


def update_association(association, association_args, args,
                       api=None, path=None, session_file=None):
    """Updates association properties

    """
    if api is None:
        api = bigml.api.BigML()

    message = dated("Updating association. %s\n" %
                    get_url(association))
    log_message(message, log_file=session_file,
                console=args.verbosity)
    association = api.update_association(association, association_args)
    check_resource_error(association, "Failed to update association: %s"
                         % association['resource'])
    association = check_resource(association,
                                 api.get_association, query_string=FIELDS_QS,
                                 raise_on_error=True)
    if is_shared(association):
        message = dated("Shared association link. %s\n" %
                        get_url(association, shared=True))
        log_message(message, log_file=session_file, console=args.verbosity)
        if args.reports:
            report(args.reports, path, association)

    return association



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2017-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from bigmler.tests.common_steps import shell_execute
from bigmler.tests.world import res_filename, ok_


def i_create_all_ts_resources_with_no_headers(
    step, data=None, output=None):
    """Step: I create BigML time series resources uploading train <data>
    file with no headers and log forecasts in <output>
    """
    ok_(data is not None and output is not None)
    command = ("bigmler time-series --train " + res_filename(data) +
               " --store --output " + output +
               " --no-train-header --no-test-header")
    shell_execute(command, output, options='--prediction-header')


def i_create_all_resources(step, data=None, test=None, output=None):
    """Step: I create BigML time series resources uploading train <data>
    file to test <test> and log forecasts in <output>
    """
    ok_(data is not None and test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler --train " + res_filename(data) + " --test " + test +
               " --store --output " + output)
    shell_execute(command, output, test=test)



# -*- coding: utf-8 -*-
#
# Copyright 2021 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer - image annotations formatting

"""

import os
import json
import sys
import xml.etree.ElementTree as ET
import re
import glob

from zipfile import ZipFile

import cv2

from bigml.constants import IMAGE_EXTENSIONS
from bigml.util import infer_field_type

import bigmler.utils as u




FILE_ATTR = "file"
BBOXES_ATTR = "boxes"


def relative_path(base_dir, absolute_path):
    """Uses the basename of the file if no base directory is given,
    and the relative path otherwise

    """
    if base_dir is None:
        return os.path.basename(absolute_path)
    return os.path.relpath(absolute_path, base_dir)


def fields_from_annotations(annotations_file):
    """Infers the type of the fields that will contain the annotations
    in an annotations file.
    """
    if not (annotations_file and os.path.exists(annotations_file)):
        sys.exit("Failed to find the annotations file.")
    with open(annotations_file) as annotations_handler:
        annotations = json.load(annotations_handler)
        annotations_fields = {}
        for annotation in annotations:
            if "file" in annotation:
                for field, value in annotation.items():
                    if field != "file" and field not in annotations_fields:
                        annotations_fields[field] = infer_field_type(
                            field, value)
    return list(annotations_fields.values())


def labels_from_annotations(annotations_file):
    """Returns the labels in each field. """
    if not (annotations_file and os.path.exists(annotations_file)):
        sys.exit("Failed to find the annotations file.")
    with open(annotations_file) as annotations_handler:
        annotations = json.load(annotations_handler)
        annotations_labels = {}
        for annotation in annotations:
            if "file" in annotation:
                for field, value in annotation.items():
                    if field != "file":
                        if field not in annotations_labels:
                            annotations_labels[field] = []
                        if isinstance(value, dict):
                            if value["label"] not in annotations_labels[field]:
                                annotations_labels[field].append(
                                    value["label"])
                        elif isinstance(value, str):
                            if value not in \
                                    annotations_labels[field]:
                                annotations_labels[field].append(value)
                        elif isinstance(value, list):
                            for item in value:
                                if item["label"] not in \
                                        annotations_labels[field]:
                                    annotations_labels[field].append(
                                        item["label"])
    return annotations_labels


def bigml_metadata(args, images_list=None, new_fields=None):
    """Creates a metadata file to summarize the locations of images and
    annotations
    """
    try:
        if args.images_file is None and args.images_dir and \
                os.path.exists(args.images_dir):
            try:
                zipname = args.images_dir.split(os.path.sep)[-1]
                if not zipname:
                    zipname = args.images_dir.split(os.path.sep)[-2]
            except Exception:
                zipname = "annotated_images"
            zip_path = os.path.join(args.output_dir, "%s.zip" % zipname)
            files = glob.glob(os.path.join(args.images_dir, "**"),
                              recursive=True)
            images_list = [filename for
                           filename in files if
                           os.path.splitext(filename)[1].lower()[1:] in
                           IMAGE_EXTENSIONS]

        if images_list:
            if not os.path.exists(zip_path):
                with ZipFile(zip_path, 'w') as zip_obj:
                    for filename in images_list:
                        zip_obj.write(filename,
                                     relative_path(args.images_dir, filename))
            args.images_file = zip_path

        output = args.images_file
        # if annotations are also uploaded
        if args.annotations_file:
            if new_fields is None:
                new_fields = fields_from_annotations(args.annotations_file)

            metafile_name = os.path.join(args.output_dir, "metadata.json")
            annotations_file = os.path.relpath(args.annotations_file,
                args.output_dir)
            images_file = os.path.relpath(args.images_file,
                args.output_dir)

            with open(metafile_name, "w") as \
                    metafile:
                meta_info = {"description": ("Transformed file for images"
                                             " annotations"),
                             "images_file": images_file,
                             "source_id": args.source,
                             "new_fields": new_fields,
                             "annotations": annotations_file}
                json.dump(meta_info, metafile)
            output = metafile_name
    except Exception as exc:
        sys.exit(f"Failed to create an annotated images upload file: {exc}")

    return output


def bigml_coco_file(args, session_file):
    """Translates from alternative annotations format, like VOC and YOLO to
    the format accepted by BigML

    """

    args.annotations_file = os.path.join(args.output_dir, "annotations.json")
    filenames = voc_to_cocojson(args.annotations_dir, args,
                                session_file) \
        if args.annotations_language == "VOC" else \
        yolo_to_cocojson(args.annotations_dir, args, session_file)
    return bigml_metadata(args, images_list=filenames,
                          new_fields=[{"name": "boxes", "optype": "regions"}])


def get_image_info(annotation_root):
    """Returns the basic image descriptors"""
    path = annotation_root.findtext('path')
    if path is None:
        filename = annotation_root.findtext('filename')
    else:
        filename = os.path.basename(path)
    img_name = os.path.basename(filename)
    img_id = os.path.splitext(img_name)[0]
    if isinstance(img_id, str):
        img_id = int(re.findall(r'\d+', img_id)[0])

    size = annotation_root.find('size')
    width = int(size.findtext('width'))
    height = int(size.findtext('height'))

    image_info = {
        'filename': filename,
        'height': height,
        'width': width,
        'id': img_id
    }
    return image_info


def get_coco_annotation_from_object(obj, filename, logfile, warnings):
    """Returns the annotation properties reading them from coco notation"""
    label = obj.findtext('name')
    bndbox = obj.find('bndbox')
    ## xmin = int(bndbox.findtext('xmin')) - 1
    xmin = int(bndbox.findtext('xmin'))
    ## ymin = int(bndbox.findtext('ymin')) - 1
    ymin = int(bndbox.findtext('ymin'))
    xmax = int(bndbox.findtext('xmax'))
    ymax = int(bndbox.findtext('ymax'))

    ## assert xmax >= xmin and ymax >= ymin, \
    ## f"Box size error !: (xmin, ymin, xmax, ymax): {xmin, ymin, xmax, ymax}"
    ## o_width = xmax - xmin
    ## o_height = ymax - ymin

    # logging possible errors
    if xmin == xmax or ymin == ymax:
        logfile.write(f"Possible bndbox error in {filename}:\n" +
                      "  min equal to max: (xmin, ymin, xmax, ymax):" +
                      f"  {xmin, ymin, xmax, ymax}\n")
        warnings += 1
    if xmin > xmax or ymin > ymax:
        logfile.write(f"Possible bndbox error in {filename}:\n" +
                      "  min greater than max: (xmin, ymin, xmax, ymax):" +
                      f"  {xmin, ymin, xmax, ymax}\n")
        warnings += 1

    annotation = {
        'label': label,
        'xmin': xmin,
        'ymin': ymin,
        'xmax': xmax,
        'ymax': ymax
    }
    return annotation, warnings


def yolo_to_cocojson(yolo_dir, args, session_file):
    """Builds annotations file from yolo separated annotation files """
    ## assume .txt extension for yolo annotation files,
    ## may be input as args
    yolo_extension = '.txt'
    ## extensions of acceptable image files

    output_json_array = []

    filenames = []

    logfile_name = args.annotations_file + ".log"
    with open(logfile_name, "w") as logfile:

        warnings = 0
        message = "Start converting YOLO files in " + yolo_dir + "\n"
        u.log_message(message, session_file, console=args.verbosity)
        logfile.write("\n%s" % message)

        # Images will be found either in the images_dir file or where
        # the yolo annotation file points to
        if args.images_dir is not None and os.path.exists(args.images_dir):
            images_dir = args.images_dir
        else:
            images_dir = yolo_dir
        filenames = glob.glob(os.path.join(images_dir, "**"),
                              recursive=True)
        filenames = [os.path.abspath(filename) for
                     filename in filenames if
                     os.path.splitext(filename)[1].lower() in
                     IMAGE_EXTENSIONS]

        ## Read yolo annotation txt file
        yolo_file_list = []
        for a_file in glob.glob(os.path.join(yolo_dir, "**"), recursive=True):
            if a_file.endswith(yolo_extension):
                # print(os.path.join(yolo_dir, a_file))
                logfile.write(a_file + "\n")
                yolo_file_list.append(os.path.abspath(a_file))

        logfile.write("\n")
        ## Check if the yolo files (.txt) in the yolo_dir have their
        ## corresponding images, .jpg, .jpeg, .png, etc
        for yolo_filename in yolo_file_list:
            filename_base = os.path.splitext(yolo_filename)[0]
            # print("filename_base: " + filename_base)
            # print(yolo_dir + "/" + filename_base + ".*")
            image_filename = ''

            matched_files = [filename for filename in
                    glob.glob(filename_base + ".*") if not
                    filename.endswith(yolo_extension)]
            if len(matched_files) == 0:
                logfile.write(" Warning: no image file for " +
                        yolo_filename +"\n")
            else:
                # print(*matched_files, sep = ", ")
                # print("matched_files length: " + str(len(matched_files)))

                ## traverse the matched file lists to find the image file,
                ## giving a warning each time a matched file is not in the
                ## image_extensions list.
                ## if there are more than one supported image file,
                ## the last one in the matched_file list is used
                for a_file in matched_files:
                    filenames.append(a_file)
                    ext = os.path.splitext(a_file)[1]
                    if ext.lower() in IMAGE_EXTENSIONS:
                        image_filename = a_file
                    else:
                        warnings += 1
                        logfile.write("Warning: unknown image file for " +
                                      yolo_filename + "\n")
                        logfile.write(a_file)
                        logfile.write("\n")

            if not image_filename or image_filename.isspace():
                continue

            # print("image_filename: " + image_filename + "\n")
            logfile.write("converting for: " + image_filename + "\n")
            image_filename_base = relative_path(images_dir, image_filename)

            #pylint: disable=locally-disabled,no-member
            image_file = cv2.imread(image_filename)
            if image_file is not None:
                if image_filename not in filenames:
                    filenames.append(image_filename)
                with open(yolo_filename, "r") as yolo_file:
                    yolo_read_lines = yolo_file.readlines()

                logfile.write("taking as filename: " +
                              image_filename_base + "\n")

                one_image_dict = {
                    ## possible args options for full path or basename
                    FILE_ATTR: image_filename_base,
                    BBOXES_ATTR: []
                }

                ## yolo format - (label, xc, yc, width, height)
                ## coco format - (label, xmin, ymin, xmax, ymax)
                for yolo_line in yolo_read_lines:
                    one_yolo_annotation = yolo_line
                    label = one_yolo_annotation.split()[0]

                    x_center = float(one_yolo_annotation.split()[1])
                    y_center = float(one_yolo_annotation.split()[2])
                    width = float(one_yolo_annotation.split()[3])
                    height = float(one_yolo_annotation.split()[4])

                    if x_center.is_integer() and y_center.is_integer() and \
                            width.is_integer() and height.is_integer():
                        round_x_center = x_center
                        round_y_center = y_center
                        proper_width = width
                        proper_height = height
                    else:
                        float_x_center = image_file.shape[1] * x_center
                        float_y_center = image_file.shape[0] * y_center
                        float_width = image_file.shape[1] * width
                        float_height = image_file.shape[0] * height

                        int_x_center = int(float_x_center)
                        int_y_center = int(float_y_center)
                        int_width = int(float_width)
                        int_height = int(float_height)

                        round_width = round(float_width)
                        round_height = round(float_height)

                        ## Yolo uses normalized coordinates to specify
                        ## the bonding boxes. [x_center, y_center, width, height]
                        ## Because they are normalized, i.e. ratios to the width
                        ## and height of the image, they stay the same when the image
                        ## is enlarged or shrank. But the bounding boxes may
                        ## lose pixels due to rounding.

                        ## The following operations try to reverse the effect of
                        ## rounding and add a pixel to the bouding box if reasonable.
                        x_center_decimal = float_x_center - int(float_x_center)
                        y_center_decimal = float_y_center - int(float_y_center)
                        #print("xc_decimal=", x_center_decimal, " yc_decimal=",
                        #        y_center_decimal, "\n")
                        logfile.write("xc_decimal=" + str(x_center_decimal) +
                                      " yc_decimal=" + str(y_center_decimal) +
                                      "\n")
                        if x_center_decimal <= 0.25:
                            round_x_center = int(float_x_center)
                        elif 0.75 >= x_center_decimal > 0.25:
                            round_x_center = int(float_x_center) + 0.5
                        else:
                            round_x_center = int(float_x_center) + 1

                        if y_center_decimal <= 0.25:
                            round_y_center = int(float_y_center)
                        elif 0.75 >= y_center_decimal > 0.25:
                            round_y_center = int(float_y_center) + 0.5
                        else:
                            round_y_center = int(float_y_center) + 1

                        if round_x_center - int(round_x_center) == 0:
                            if (round(float_width) % 2) == 0:
                                proper_width = round(float_width)
                            else:
                                proper_width = round(float_width) + 1
                        else:
                            if (round(float_width) % 2) == 0:
                                proper_width = round(float_width) + 1
                            else:
                                proper_width = round(float_width)

                        if round_y_center - int(round_y_center) == 0:
                            if (round(float_height) % 2) == 0:
                                proper_height = round(float_height)
                            else:
                                proper_height = round(float_height) + 1
                        else:
                            if (round(float_height) % 2) == 0:
                                proper_height = round(float_height) + 1
                            else:
                                proper_height = round(float_height)


                        #print("xc=", x_center, " yc=", y_center, " w=", width,
                        #      " h=", height, "\n")
                        logfile.write("xc=" + str(x_center) + " yc=" + str(y_center) +
                                    " w=" + str(width) + " h=" + str(height) + "\n")
                        #print("float_xc=", float_x_center, " float_yc=", float_y_center,
                        #      " float_w=", float_width, " float_h=", float_height, "\n")
                        logfile.write("float_xc=" + str(float_x_center) + " float_yc=" +
                                      str(float_y_center) + " float_w=" +
                                      str(float_width) + " float_h=" +
                                      str(float_height) + "\n")
                        #print("round_xc=", round_x_center, " round_yc=", round_y_center,
                        #      " round_w=", round_width, " round_h=", round_height, "\n")
                        logfile.write("round_xc=" + str(round_x_center) + " round_yc=" +
                                      str(round_y_center) + " round_w=" +
                                      str(round_width) + " round_h=" +
                                      str(round_height) + "\n")
                        #print("round_xc=", round_x_center, " round_yc=", round_y_center,
                        #    " proper_w=", proper_width, " proper_h=", proper_height, "\n")
                        logfile.write("round_xc=" + str(round_x_center) + " round_yc=" +
                                      str(round_y_center) + " proper_w=" +
                                      str(proper_width) + " proper_h=" +
                                      str(proper_height) + "\n")
                        #print("int_xc=", int_x_center, " int_yc=", int_y_center,
                        #      " int_w=", int_width, " int_h=", int_height, "\n\n")
                        logfile.write("int_xc=" + str(int_x_center) + " int_yc=" +
                                      str(int_y_center) + " int_w=" + str(int_width) +
                                      " int_h=" + str(int_height) + "\n\n")

                    ## x_min = int_x_center - int_width/2
                    x_min = int(round_x_center - proper_width/2)
                    ## y_min = int_y_center - int_height/2
                    y_min = int(round_y_center - proper_height/2)
                    ## x_max = x_min + int_width
                    x_max = int(x_min + proper_width)
                    ## y_max = y_min + int_height
                    y_max = int(y_min + proper_height)
                    annotation = {
                            'label': label,
                            'xmin': x_min,
                            'ymin': y_min,
                            'xmax': x_max,
                            'ymax': y_max
                            }
                    one_image_dict[BBOXES_ATTR].append(annotation)

                # output_json_dict['annotations'].append(one_image_dict)
                output_json_array.append(one_image_dict)
            else:
                warnings += 1
                logfile.write("Warning: failed to find image file for " +
                              image_file + "\n")
                logfile.write("\n")


    if warnings > 0:
        message = f"\nThere are {warnings} warnings, " \
                  f"see the log file {logfile_name}\n"
        u.log_message(message, session_file, console=args.verbosity)

    with open(args.annotations_file, 'w') as handler:
        json.dump(output_json_array, handler, indent=2)

    return [relative_path(images_dir, filename) for filename in \
        filenames]


def voc_to_cocojson(voc_dir, args, session_file):
    """Translates annotations from a VOC format, where each image is associated
    to a .xml file that contains one object per associated info. It returns
    the list of images it refers to.

    """

    output_json_array = []

    filenames = []

    annotation_file_list = []
    for file in os.listdir(voc_dir):
        if file.endswith(".xml"):
            annotation_file_list.append(os.path.join(voc_dir, file))
    logfile_name = args.annotations_file + ".log"
    with open(logfile_name, "w") as logfile:

        ## Start bounding_box_id, possible future use, may be input as args
        bndbox_id = 1
        warnings = 0
        message = "Start converting VOC files in " + voc_dir + "\n"
        u.log_message(message, session_file, console=args.verbosity)
        logfile.write("\n\n%s\n" % message)

        # Images will be found either in the images_dir file or where
        # the yolo annotation file points to
        if args.images_dir is not None and os.path.exists(args.images_dir):
            filenames = glob.glob(os.path.join(args.images_dir, "**"),
                                  recursive=True)
            filenames = [os.path.abspath(filename) for
                         filename in filenames if
                         os.path.splitext(filename)[1].lower() in
                         IMAGE_EXTENSIONS]

        for a_file in annotation_file_list:
            ## Read annotation xml per image
            annotation_tree = ET.parse(a_file)
            annotation_root = annotation_tree.getroot()

            img_info = get_image_info(annotation_root)
            folder = annotation_root.findtext('folder') or ''
            base_dir = os.getcwd()
            if args.images_dir is not None:
                base_dir = os.path.join(base_dir, args.images_dir)
            image_filename_base = os.path.join(folder, img_info['filename'])
            if args.images_dir is not None:
                filename = os.path.join(base_dir, image_filename_base)

            if os.path.exists(filename):
                if filename not in filenames:
                    filenames.append(filename)

                ## possible args options for full path or basename
                logfile.write("converting for: " + filename + "\n")
                logfile.write("taking as filename: " + image_filename_base + "\n")

                one_image_dict = {
                    ## possible args options for full path or basename
                    FILE_ATTR: image_filename_base,
                    BBOXES_ATTR: []
                }

                for obj in annotation_root.findall('object'):
                    annotation, warnings = get_coco_annotation_from_object( \
                        obj, filename, logfile, warnings)
                    one_image_dict[BBOXES_ATTR].append(annotation)
                    bndbox_id = bndbox_id + 1

                output_json_array.append(one_image_dict)
            else:
                warnings += 1
                logfile.write("failed to find: " + filename + "\n")

    if warnings > 0:
        message = f"\nThere are {warnings} warnings, " \
                f"see the log file {logfile_name}\n"
        u.log_message(message, session_file, console=args.verbosity)

    with open(args.annotations_file, 'w') as handler:
        json.dump(output_json_array, handler, indent=2)

    return [relative_path(args.images_dir, filename) for filename in \
        filenames]



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2019-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os

from bigml.api import check_resource

from bigmler.tests.common_steps import shell_execute
from bigmler.tests.world import world, res_filename, ok_


def i_create_all_pca_resources_with_no_headers(step, data=None, test=None, output=None):
    """Step: I create BigML PCA resources uploading train <data> file with no
    headers to test <test> with no headers and log projections in <output>
    """
    ok_(data is not None and test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler pca --train " + res_filename(data) +
               " --test " + test +
               " --store --output " + output +
               " --no-train-header --no-test-header")
    shell_execute(command, output, test=test, options="--projection-header")


def i_create_all_pca_resources(step, data=None, test=None, output=None):
    """Step: I create BigML PCA resources uploading train <data> file to test
    <test> and log predictions in <output>
    """
    ok_(data is not None and test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler pca --train " + res_filename(data) +
               " --test " + test +
               " --store --output "
               + output)
    shell_execute(command, output, test=test)


def i_create_pca_resources_from_dataset(step, test=None, output=None):
    """Step: I create BigML PCA resources using dataset to test <test> and log
    predictions in <output>
    """
    ok_(test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler pca --dataset " +
               world.dataset['resource'] + " --test " + test +
               " --store --output " + output)
    shell_execute(command, output, test=test)


def i_create_pca_resources_from_source(
    step, test=None, output=None):
    """Step: I create BigML PCA resources using source to test <test>
    and log predictions in <output>
    """
    ok_(test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler pca --source " + world.source['resource']
               + " --test " + test + " --store --output " + output)
    shell_execute(command, output, test=test)


def i_create_pca_resources_from_model(step, test=None, output=None):
    """Step: I create BigML PCA resources using model to test <test> and log
    predictions in <output>
    """
    ok_(test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler pca --pca " +
               world.pca['resource'] + " --test " +
               test + " --store --output " +
               output)
    shell_execute(command, output, test=test)


def i_create_pca_resources_from_model_remote(step, test=None, output=None):
    """Step: I create BigML PCA resources using model to test <test> as batch
    prediction and log predictions in <output>
    """
    ok_(test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler pca --pca " +
               world.pca['resource'] + " --test " + test +
               " --store --remote --output " +
               output)
    shell_execute(command, output, test=test, options="--no-header")


def i_check_create_pca_model(step):
    """Step: I check that the pca model has been created"""
    pca_file = os.path.join(world.directory, "pcas")
    message = None
    try:
        with open(pca_file) as handler:
            pca = check_resource(handler.readline().strip(),
                                 world.api.get_pca)
            world.pcas.append(pca['resource'])
            world.pca = pca
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2014-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

from bigml.api import check_resource

from bigmler.tests.common_steps import shell_execute
from bigmler.tests.world import world, res_filename, ok_


def i_check_create_batch_prediction(step):
    """Step: I check that the batch prediction has been created"""
    batch_prediction_file = os.path.join(world.directory, "batch_prediction")
    message = None
    try:
        with open(batch_prediction_file) as handler:
            batch_prediction = check_resource(handler.readline().strip(),
                                              world.api.get_batch_prediction)
        world.batch_predictions.append(batch_prediction['resource'])
        world.batch_prediction = batch_prediction
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_create_batch_projection(step):
    """Step: I check that the batch projection has been created"""
    batch_projection_file = os.path.join(world.directory, "batch_projection")
    message = None
    try:
        with open(batch_projection_file) as handler:
            batch_projection = check_resource(handler.readline().strip(),
                                              world.api.get_batch_projection)
        world.batch_projections.append(batch_projection['resource'])
        world.batch_projection = batch_projection
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_create_test_source(step):
    """Step: I check that the source has been created from the test file"""
    test_source_file = os.path.join(world.directory, "source_test")
    message = None
    try:
        with open(test_source_file) as handler:
            test_source = check_resource(handler.readline().strip(),
                                         world.api.get_source)
        world.sources.append(test_source['resource'])
        world.test_source = test_source
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_create_test_dataset(step):
    """Step: I check that the dataset has been created from the test file"""
    test_dataset_file = os.path.join(world.directory, "dataset_test")
    message = None
    try:
        with open(test_dataset_file) as handler:
            test_dataset = check_resource(handler.readline().strip(),
                                          world.api.get_dataset)
        world.datasets.append(test_dataset['resource'])
        world.test_lines = test_dataset['object']['rows'] + world.test_header
        world.test_dataset = test_dataset
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_create_batch_centroid(step):
    """Step: I check that the batch centroid prediction has been created"""
    batch_prediction_file = os.path.join(world.directory, "batch_centroid")
    message = None
    try:
        with open(batch_prediction_file) as handler:
            batch_centroid = check_resource(handler.readline().strip(),
                                            world.api.get_batch_centroid)
        world.batch_centroids.append(batch_centroid['resource'])
        world.batch_centroid = batch_centroid
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_create_batch_anomaly_scores(step):
    """Step: I check that the batch anomaly scores prediction has been
    created
    """
    batch_prediction_file = os.path.join(world.directory,
        "batch_anomaly_score")
    message = None
    try:
        with open(batch_prediction_file) as handler:
            batch_anomaly_score = check_resource(
                handler.readline().strip(), world.api.get_batch_anomaly_score)
        world.batch_anomaly_scores.append(batch_anomaly_score['resource'])
        world.batch_anomaly_score = batch_anomaly_score
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_create_batch_predictions_dataset(step):
    """Step: I check that the batch predictions dataset exists"""
    dataset_file = os.path.join(world.directory, "batch_prediction_dataset")
    message = None
    try:
        with open(dataset_file) as handler:
            dataset = check_resource(handler.readline().strip(),
                                     api=world.api)
            world.datasets.append(dataset['resource'])
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_create_batch_centroids_dataset(step):
    """Step: I check that the batch centroids dataset exists"""
    dataset_file = os.path.join(world.directory, "batch_centroid_dataset")
    message = None
    try:
        with open(dataset_file) as handler:
            dataset = check_resource(handler.readline().strip(),
                                     api=world.api)
        world.datasets.append(dataset['resource'])
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_create_resources_from_model_with_op_remote(step, operating_point=None,
                                                 test=None, output=None):
    """Step: I create BigML resources using model with operating point
    <operating_point> to test <test> and log predictions in <output>
    """
    ok_(operating_point is not None and
        test is not None and output is not None)
    test = res_filename(test)
    operating_point = res_filename(operating_point)
    command = ("bigmler --model " + world.model['resource'] + " --test " +
               test + " --operating-point " + operating_point +
               " --store --remote --output " + output +
               " --max-batch-models 1")
    shell_execute(command, output, test=test, options="--no-header")


def i_create_resources_from_model_remote_no_batch(step,
                                                  test=None, output=None):
    """Step: I create BigML remote predictions one by one using model
    to test <test> and log predictions in <output>
    """
    ok_(test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler --model " + world.model['resource'] + " --test " +
               test + " --no-batch --store --remote --output " + output +
               " --max-batch-models 1")
    shell_execute(command, output, test=test, options="--no-header")



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2015-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

""" Testing delete subcommand, --from-dir option

"""
from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)

import bigmler.tests.delete_subcommand_steps as test_delete


def setup_module():
    """Setup for the module

    """
    common_setup_module()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()

class TestDeleteDir:
    """Testing delete from directory"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def test_scenario1(self):
        """
        Scenario: Sucessfully deleting resources from a directory:
            Given I store the number of existing resources
            And I create BigML resources uploading train "<data>" storing results in "<output_dir>"
            And I check that the number of resources has changed
            And I delete the resources from the output directory
            Then the number of resources has not changed

            Examples:
            | data               | output_dir
            | ../data/iris.csv   | ./scenario_del_10
        """
        print(self.test_scenario1.__doc__)
        headers = ["data", "output_dir"]
        examples = [
            ['data/iris.csv', 'scenario_del_10']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_delete.i_store_the_number_of_resources(self)
            test_delete.i_create_source_from_file(
                self, data=example["data"], output_dir=example["output_dir"])
            test_delete.i_check_changed_number_of_resources(self)
            test_delete.i_delete_resources_from_dir(self)
            test_delete.i_check_equal_number_of_resources(self)



# -*- coding: utf-8 -*-
#
# Copyright 2016-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer - topic model subcommand processing dispatching

"""


import sys
import os

import bigml.api

import bigmler.utils as u
import bigmler.resourcesapi.common as r
import bigmler.resourcesapi.topic_models as rtm
import bigmler.resourcesapi.batch_topic_distributions as rtd
import bigmler.pre_model_steps as pms
import bigmler.processing.args as a
import bigmler.processing.sources as ps
import bigmler.processing.datasets as pd
import bigmler.processing.topicmodels as pt

from bigmler.resourcesapi.datasets import set_basic_dataset_args
from bigmler.defaults import DEFAULTS_FILE
from bigmler.reports import clear_reports, upload_reports
from bigmler.topicdistribution import (topic_distribution,
                                       remote_topic_distribution)
from bigmler.command import get_context
from bigmler.dispatcher import SESSIONS_LOG, clear_log_files, get_test_dataset

COMMAND_LOG = ".bigmler_topic_model"
DIRS_LOG = ".bigmler_topic_model_dir_stack"
LOG_FILES = [COMMAND_LOG, DIRS_LOG, u.NEW_DIRS_LOG]
MINIMUM_MODEL = "full=false"
DEFAULT_OUTPUT = "topic_distributions.csv"

SETTINGS = {
    "command_log": COMMAND_LOG,
    "sessions_log": SESSIONS_LOG,
    "dirs_log": DIRS_LOG,
    "default_output": DEFAULT_OUTPUT,
    "defaults_file": DEFAULTS_FILE}


#pylint: disable=locally-disabled,dangerous-default-value
def topic_model_dispatcher(args=sys.argv[1:]):
    """Parses command line and calls the different processing functions

    """

    # If --clear-logs the log files are cleared
    if "--clear-logs" in args:
        clear_log_files(LOG_FILES)

    command_args, _, api, session_file, resume = get_context(args, SETTINGS)

    # Selects the action to perform
    if a.has_train(command_args) or a.has_test(command_args):
        output_args = a.get_output_args(api, command_args, resume)
        compute_output(**output_args)
    u.log_message("_" * 80 + "\n", log_file=session_file)


def compute_output(api, args):
    """ Creates one or more models using the `training_set` or uses the ids
    of previously created BigML models to make predictions for the `test_set`.

    """

    topic_model = None
    topic_models = None
    # no multi-label support at present

    # variables from command-line options
    resume = args.resume_
    topic_model_ids = args.topic_model_ids_
    output = args.predictions
    # there's only one topic model resource to be generated at present
    args.max_parallel_topic_models = 1
    # topic models cannot be published yet.
    args.public_topic_model = False

    # It is compulsory to have a description to publish either datasets or
    # topic models
    if (not args.description_ and (args.public_topic_model or
                                   args.public_dataset)):
        sys.exit("You should provide a description to publish.")

    # When using --new-fields, it is compulsory to specify also a dataset
    # id
    if args.new_fields and not args.dataset:
        sys.exit("To use --new-fields you must also provide a dataset id"
                 " to generate the new dataset from it.")

    path = u.check_dir(output)
    session_file = "%s%s%s" % (path, os.sep, SESSIONS_LOG)
    csv_properties = {}
    # If logging is required set the file for logging
    log = None
    if args.log_file:
        u.check_dir(args.log_file)
        log = args.log_file
        # If --clear_logs the log files are cleared
        clear_log_files([log])

    # basic pre-model step: creating or retrieving the source related info
    source, resume, csv_properties, fields = pms.get_source_info(
        api, args, resume, csv_properties, session_file, path, log)
    # basic pre-model step: creating or retrieving the dataset related info
    dataset_properties = pms.get_dataset_info(
        api, args, resume, source,
        csv_properties, fields, session_file, path, log)
    (_, datasets, test_dataset,
     resume, csv_properties, fields) = dataset_properties
    if args.topic_model_file:
        # topic model is retrieved from the contents of the given local JSON
        # file
        topic_model, csv_properties, fields = u.read_local_resource(
            args.topic_model_file,
            csv_properties=csv_properties)
        topic_models = [topic_model]
        topic_model_ids = [topic_model['resource']]
    else:
        # topic model is retrieved from the remote object
        topic_models, topic_model_ids, resume = pt.topic_model_processing(
            datasets, topic_models, topic_model_ids, api, args, resume,
            fields=fields,
            session_file=session_file, path=path, log=log)
        if topic_models:
            topic_model = topic_models[0]

    # We update the topic model's public state if needed
    if topic_model:
        if isinstance(topic_model, str):
            if not a.has_test(args):
                query_string = MINIMUM_MODEL
            else:
                query_string = ''
            topic_model = u.check_resource(topic_model, api.topic_model,
                                           query_string=query_string)
        topic_models[0] = topic_model
        if (args.public_topic_model or
                (args.shared_flag and
                 r.shared_changed(args.shared, topic_model))):
            topic_model_args = {}
            if args.shared_flag and \
                    r.shared_changed(args.shared, topic_model):
                topic_model_args.update(shared=args.shared)
            if args.public_topic_model:
                topic_model_args.update(rtm.set_publish_topic_model_args(args))
            if topic_model_args:
                topic_model = rtm.update_topic_model( \
                    topic_model, topic_model_args, args,
                    api=api, path=path,
                    session_file=session_file)
                topic_models[0] = topic_model

    # We get the fields of the topic model if we haven't got
    # them yet and need them
    if topic_model and args.test_set:
        csv_properties.update({'objective_field_present': False,
                               'objective_field': None})
        fields = pt.get_topic_model_fields(topic_model, csv_properties, args)

    # If predicting
    if topic_models and (a.has_test(args) or (test_dataset and args.remote)):
        if test_dataset is None:
            test_dataset = get_test_dataset(args)

        # Remote topic distributions:topic distributions are computed as
        # batch topic distributions
        # in bigml.com except when --no-batch flag is set.
        if args.remote and not args.no_batch:
            # create test source from file
            test_name = "%s - test" % args.name
            if args.test_source is None:
                test_properties = ps.test_source_processing(
                    api, args, resume, name=test_name,
                    session_file=session_file, path=path, log=log)
                (test_source, resume,
                 csv_properties, test_fields) = test_properties
            else:
                test_source_id = bigml.api.get_source_id(args.test_source)
                test_source = api.check_resource(test_source_id)
            if test_dataset is None:
                # create test dataset from test source
                dataset_args = set_basic_dataset_args(args, name=test_name)
                test_dataset, resume = pd.alternative_dataset_processing(
                    test_source, "test", dataset_args, api, args,
                    resume, session_file=session_file, path=path, log=log)
            else:
                test_dataset_id = bigml.api.get_dataset_id(test_dataset)
                test_dataset = api.check_resource(test_dataset_id)
            test_fields = pd.get_fields_structure(test_dataset,
                                                  csv_properties)
            batch_topic_distribution_args = \
                rtd.set_batch_topic_distribution_args( \
                args, fields=fields, \
                dataset_fields=test_fields)

            remote_topic_distribution( \
                topic_model, test_dataset, batch_topic_distribution_args,
                args, api, resume, prediction_file=output,
                session_file=session_file, path=path, log=log)
        else:
            topic_distribution(topic_models, fields, args,
                               session_file=session_file)

    u.print_generated_files(path, log_file=session_file,
                            verbosity=args.verbosity)
    if args.reports:
        clear_reports(path)
        if args.upload:
            upload_reports(args.reports, path)



# -*- coding: utf-8 -*-
#
# Copyright 2020-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Resources management functions

"""



import sys

import bigml.api

from bigmler.utils import (dated, log_message, check_resource,
                           plural, get_url, is_shared,
                           check_resource_error, log_created_resources)
from bigmler.reports import report

from bigmler.resourcesapi.common import set_basic_args, \
    update_json_args, wait_for_available_tasks


def set_sample_args(args, name=None):
    """Return sample arguments dict

    """
    if name is None:
        name = args.name

    sample_args = set_basic_args(args, name)

    if 'sample' in args.json_args:
        update_json_args(sample_args, args.json_args.get('sample'))
    return sample_args


def create_samples(datasets, sample_ids, sample_args,
                   args, api=None, path=None,
                   session_file=None, log=None):
    """Create remote samples

    """
    if api is None:
        api = bigml.api.BigML()

    samples = sample_ids[:]
    existing_samples = len(samples)
    sample_args_list = []
    datasets = datasets[existing_samples:]
    # if resuming and all samples were created, there will be no datasets left
    if datasets:
        if isinstance(sample_args, list):
            sample_args_list = sample_args

        # Only one sample per command, at present
        number_of_samples = 1
        max_parallel_samples = 1
        message = dated("Creating %s.\n" %
                        plural("sample", number_of_samples))
        log_message(message, log_file=session_file,
                    console=args.verbosity)

        inprogress = []
        for i in range(0, number_of_samples):
            wait_for_available_tasks(inprogress, max_parallel_samples,
                                     api, "sample")
            if sample_args_list:
                sample_args = sample_args_list[i]

            sample = api.create_sample(datasets[i], sample_args, retries=None)
            sample_id = check_resource_error(sample,
                                             "Failed to create sample: ")
            log_message("%s\n" % sample_id, log_file=log)
            sample_ids.append(sample_id)
            inprogress.append(sample_id)
            samples.append(sample)
            log_created_resources("samples", path, sample_id, mode='a')

        if args.verbosity:
            if bigml.api.get_status(sample)['code'] != bigml.api.FINISHED:
                try:
                    sample = check_resource(sample, api.get_sample,
                                            raise_on_error=True)
                except Exception as exception:
                    sys.exit("Failed to get a finished sample: %s" %
                             str(exception))
                samples[0] = sample
            message = dated("Sample created: %s\n" %
                            get_url(sample))
            log_message(message, log_file=session_file,
                        console=args.verbosity)
            if args.reports:
                report(args.reports, path, sample)

    return samples, sample_ids


def update_sample(sample, sample_args, args,
                  api=None, path=None, session_file=None):
    """Updates sample properties

    """
    if api is None:
        api = bigml.api.BigML()
    message = dated("Updating sample. %s\n" %
                    get_url(sample))
    log_message(message, log_file=session_file,
                console=args.verbosity)
    sample = api.update_sample(sample, sample_args)
    check_resource_error(sample, "Failed to update sample: %s"
                         % sample['resource'])
    sample = check_resource(sample, api.get_sample, raise_on_error=True)
    if is_shared(sample):
        message = dated("Shared sample link. %s\n" %
                        get_url(sample, shared=True))
        log_message(message, log_file=session_file, console=args.verbosity)
        if args.reports:
            report(args.reports, path, sample)

    return sample


def get_samples(sample_ids, args,
                api=None, session_file=None, query_string=''):
    """Retrieves remote samples in its actual status

    """
    if api is None:
        api = bigml.api.BigML()
    sample_id = ""
    samples = sample_ids
    sample_id = sample_ids[0]
    message = dated("Retrieving %s. %s\n" %
                    (plural("sample", len(sample_ids)),
                     get_url(sample_id)))
    log_message(message, log_file=session_file, console=args.verbosity)
    # only one sample to predict at present
    try:
        sample = api.get_sample(sample_ids[0],
                                query_string=query_string)
        check_resource_error(sample, "Failed to create sample: %s"
                             % sample['resource'])
        sample = check_resource(sample, api=api, query_string=query_string,
                                raise_on_error=True)
    except Exception as exception:
        sys.exit("Failed to get a finished sample: %s" % str(exception))
    samples[0] = sample

    return samples, sample_ids


def set_publish_sample_args(args):
    """Set args to publish sample

    """
    public_sample = {}
    if args.public_sample:
        public_sample = {"private": False}
    return public_sample



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2017-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.



import os

from bigmler.tests.common_steps import shell_execute
from bigmler.export.dispatcher import EXTENSIONS
from bigmler.tests.world import world, res_filename, ok_, eq_


def extract_content(file_handler):
    """Reads the file and removes the model ID in the comments"""
    content = file_handler.read()
    variable_comment = " from" \
        " model/"
    try:
        position = content.index(variable_comment)
        content = content[:position] + \
            content[position + len(variable_comment) + 24:]
    except ValueError:
        pass
    return content


def i_export_model(step, language=None, output=None):
    """Step: I export the model as a function in <output>"""
    ok_(language is not None and output is not None)
    output_dir = world.directory
    command = ("bigmler export --language " + language +
               " --output-dir " + output_dir + " --model " +
               world.model['resource'])
    shell_execute(command, output)

def i_create_all_resources_to_model_with_source_attrs(
    self, data=None, source_attributes=None, output=None):
    """Step: I create BigML resources uploading train <data> file using
    <source_attributes> and log in <output>
    """
    ok_(data is not None and source_attributes is not None
        and output is not None)
    if source_attributes != "":
        source_attributes = " --source-attributes " + \
            res_filename(source_attributes)
    command = ("bigmler --train " + res_filename(data) +
               " --output " + output + source_attributes +
               " --store --max-batch-models 1 --no-fast")
    shell_execute(command, output)


def i_check_if_the_output_is_like_expected_file(step, language, expected_file):
    """Step: I check the output for <language> is like <expected_file>
    expected file
    """
    ok_(language is not None and expected_file is not None)
    with open(res_filename(expected_file)) as file_handler:
        expected_content = extract_content(file_handler)
    with open(os.path.join(world.directory, "%s.%s" % \
            (world.model['resource'].replace("/", "_"),
             EXTENSIONS[language]))) \
            as file_handler:
        output = extract_content(file_handler)
    if output.strip() != expected_content.strip():
        try:
            name, extension = expected_file.split(".")
            expected_file = "%s_p3.%s" % (name, extension)
            out_file = "%s.%s.out" % (name, extension)
            with open(res_filename(out_file), "w") as writer:
                writer.write(output)
            with open(res_filename(expected_file)) as file_handler:
                expected_content = extract_content(file_handler)
        except IOError:
            pass
    eq_(output.strip(), expected_content.strip(),
        msg="Found: %s\nExpected:%s" % \
        (world.directory + "/" + world.model["resource"],
         expected_file))



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2014-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

from bigml.api import check_resource

from bigmler.tests.common_steps import shell_execute
from bigmler.tests.world import world, res_filename, ok_


def i_create_all_mc_resources_from_dataset_with_model_fields(
    step, max_categories=None, objective=None, model_fields=None,
    test=None, output=None):
    """Step: I create BigML resources with <max_categories> as categories limit
    and <objective> as objective field and model fields <model_fields> using
    dataset to test <test> and log predictions in <output>"""
    ok_(max_categories is not None and test is not None and output is not None
        and model_fields is not None)
    command = ("bigmler --dataset " + world.dataset['resource'] +
               " --max-categories " + max_categories + " --objective " +
               objective + " --test " + test + " --store --output " +
               output + " --model-fields \"" + model_fields + "\"")
    shell_execute(command, output, test=test)


def i_create_all_mc_resources(
    step, data, max_categories=None, objective=None, test=None, output=None):
    """Step: I create BigML resources from <data> with <max_categories> as
    categories limit and <objective> as objective field to test <test> and log
    predictions in <output>
    """
    ok_(max_categories is not None and test is not None and output is not None
        and objective is not None)
    test = res_filename(test)
    command = ("bigmler --train " + res_filename(data) + " --max-categories " +
               max_categories + " --objective " + objective + " --test " +
               test + " --store --output " + output)
    shell_execute(command, output, test=test)


def i_create_all_mc_resources_from_source(
    step, max_categories=None, objective=None, test=None, output=None):
    """Step: I create BigML resources with <max_categories> as categories
    limit and <objective> as objective field using source to test <test> and
    log predictions in <output>
    """
    ok_(max_categories is not None and test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler --source " + world.source['resource'] +
               " --max-categories " + max_categories +  " --objective " +
               objective + " --test " + test + " --store --output " +
               output)
    shell_execute(command, output, test=test)


def i_create_all_mc_resources_from_dataset(
    step, max_categories=None, objective=None, test=None, output=None):
    """Step: I create BigML resources with <max_categories> as categories limit
    and <objective> as objective field using dataset to test <test> and
    log predictions in <output>
    """
    ok_(max_categories is not None and test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler --dataset " + world.dataset['resource'] +
               " --max-categories " + max_categories + " --objective " +
               objective + " --test " + test + " --store --output " +
               output)
    shell_execute(command, output, test=test)


def i_create_all_mc_resources_from_models(
    step, models_file=None, test=None, output=None):
    """Step: I create BigML resources using models in file <models_file> to
    test <test> and log predictions with combine method in <output>
    """
    ok_(models_file is not None and test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler --models " + models_file +
               " --method combined --test " + test + " --store --output "
               + output)
    shell_execute(command, output, test=test)


def i_check_create_max_categories_datasets(step):
    """Step: I check that the max_categories datasets have been created"""
    dataset_file = os.path.join(world.directory, "dataset_parts")
    message = None
    #pylint: disable=locally-disabled,import-outside-toplevel
    try:
        with open(dataset_file) as handler:
            number_of_datasets = 0
            for line in handler:
                dataset_id = line.strip()
                dataset = check_resource(dataset_id,
                                         world.api.get_dataset)
                ok_('user_metadata' in dataset['object'] and 'max_categories'
                    in dataset['object']['user_metadata'])
                world.datasets.append(dataset['resource'])
                number_of_datasets += 1
        world.number_of_models = number_of_datasets
    except Exception:
        import traceback
        message = traceback.format_exc()
    ok_(message is None, msg=message)



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2015-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing associations

"""

from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)


import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.basic_association_steps as test_association


def setup_module():
    """Setup for the module

    """
    common_setup_module()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()

class TestAssociation:
    """Testing associaton commands"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def test_scenario1(self):
        """
        Scenario: Successfully building association from scratch:
            Given I create BigML association uploading train "<data>" file and log resources in "<output_dir>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the association has been created
        """
        print(self.test_scenario1.__doc__)
        headers = ["data", "output_dir"]
        examples = [
            ['data/spam.csv', 'scenario_ass_1_r'],
            ['data/movies.csv', 'scenario_ass_1_i'],
            ['data/iris.csv', 'scenario_ass_1']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_association.i_create_association(
                self, data=example["data"], output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_association(self)

    def test_scenario2(self):
        """
        Scenario: Successfully building association from source
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML association using source and log resources in "<output_dir>"
            And I check that the dataset has been created
            And I check that the association has been created
        """
        print(self.test_scenario2.__doc__)
        headers = ["scenario", "kwargs", "output_dir"]
        examples = [
            ['scenario_ass_1',
             '{"data": "data/iris.csv", "output_dir": "scenario_ass_1"}',
             'scenario_ass_2']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_association.i_create_association_from_source(
                self, output_dir=example["output_dir"])
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_association(self)

    def test_scenario3(self):
        """
        Scenario: Successfully building test predictions from dataset
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML association using dataset and log predictions in "<output_dir>"
            And I check that the association has been created
        """
        print(self.test_scenario3.__doc__)
        headers = ["scenario", "kwargs", "output_dir"]
        examples = [
            ['scenario_ass_1',
             '{"data": "data/iris.csv", "output_dir": "scenario_ass_1"}',
             'scenario_ass_3']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_association.i_create_association_from_dataset(
                self, output_dir=example["output_dir"])
            test_pred.i_check_create_association(self)



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2015-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.



import os


from bigml.api import check_resource

from bigmler.tests.common_steps import shell_execute
from bigmler.tests.world import world, res_filename, ok_

def i_create_association(step, data=None, output_dir=None):
    """Step: I create BigML association uploading train <data> file and log
    resources in <output_dir>"""
    ok_(data is not None and output_dir is not None)
    command = ("bigmler association --train " + res_filename(data) +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "x.tmp"))


def i_check_create_association(step):
    """Step: I check that the association has been created"""
    association_file = os.path.join(world.directory, "associations")
    message = None
    try:
        with open(association_file) as handler:
            association = check_resource(handler.readline().strip(),
                                         world.api.get_association)
            world.associations.append(association['resource'])
            world.association = association
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_create_association_from_dataset(step, output_dir=None):
    """Step: I create BigML association using dataset and log resources
    in <output_dir>
    """
    ok_(output_dir is not None)
    command = ("bigmler association --dataset " +
               world.dataset['resource'] +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "x.tmp"))


def i_create_association_from_source(step, output_dir=None):
    """Step: I create BigML association using source and log resources in
    <output_dir>
    """
    ok_(output_dir is not None)
    command = ("bigmler association --source " +
               world.source['resource'] +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "x.tmp"))
